<?php

namespace FiloBlu\Flow\Model\From;

use Exception;
use FiloBlu\Flow\Helper\Data;
use Magento\Catalog\Model\Product\Action;
use Magento\Catalog\Model\ProductFactory;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Model\Context;
use Magento\Framework\Registry;
use Magento\Store\Model\StoreManagerInterface;
use Monolog\Logger;

/**
 * Class Price
 * @package FiloBlu\Flow\Model\From
 */
class Price extends AbstractFrom
{
    /**
     * @var ProductFactory
     */
    protected $_productFactory;
    /**
     * @var mixed
     */
    protected $_productAction;
    /**
     * @var mixed
     */
    protected $_configurableAction;
    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var Data
     */
    protected $_helper_flow;

    /**
     * Price constructor.
     * @param Context $context
     * @param Registry $registry
     * @param Data $helper_flow
     * @param StoreManagerInterface $storeManager
     * @param ProductFactory $productFactory
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        Data $helper_flow,
        StoreManagerInterface $storeManager,
        ProductFactory $productFactory,
        array $data = []
    ) {
        $this->_helper_flow = $helper_flow;
        $this->_storeManager = $storeManager;
        $this->_productFactory = $productFactory;
        $this->_productAction = ObjectManager::getInstance()->get(Action::class);
        $this->_configurableAction = ObjectManager::getInstance()->create(Configurable::class);
        parent::__construct($context, $registry, null, null, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init(\FiloBlu\Flow\Model\ResourceModel\From\Price::class);
    }

    /**
     * @return $this|bool|mixed
     * @throws Exception
     */
    public function process()
    {
        // Floatval will return zero if fails (even with ctype_space)
        // With zero this process will skip the import
        $prezzo = floatval($this->getPrezzo());
        $special_price = floatval($this->getData('special_price'));
        // If special price is less than zero then i need to remove it from Magento passing '' as value
        $special_price = ($special_price < 0 ? '' : $special_price);
        // Do not allow prices below zero
        $prezzo = ($prezzo < 0 ? 0 : $prezzo);

        $price_imported = $this->setPriceByPriceAttributeName($prezzo, 'price');
        $special_price_imported = $this->setPriceByPriceAttributeName($special_price, 'special_price');

        // Write error only if both fails
        if(!$price_imported && !$special_price_imported){
            $message = sprintf("Price is zero or not valid! Skipping! product_code: '%s'", $this->getSku());
            return $this->error(false, $message);
        }
        return $this;
    }

    public function setPriceByPriceAttributeName($prezzo, $price_name){

        if (is_numeric($prezzo) && $prezzo == 0){
            return false;
        }

        $productSku = $this->getSku();
        $paese = $this->getPaese();

        $allStore = $this->_helper_flow->getStoreInWebsiteId();

        if ($paese === null) {
            $paese = 'admin';
        }
        $storeId = $allStore[$paese]['store_id'];

        $productId = $this->_productFactory->create()->getIdBySku($productSku);

        if (!$productId) {
            $message = sprintf("Product not found! product_code: '%s'", $this->getSku());
            return $this->error(false, $message);
        }

        if ($productId) {
            $pids[] = $productId;

            // salvo il prezzo sul prodotto configurabile padre
            $parent_product_id = $this->_configurableAction->getParentIdsByChild($productId);

            if (!empty($parent_product_id)) {
                $pids = array_merge($pids, $parent_product_id);
            }

            try {
                $this->_productAction->updateAttributes($pids, [$price_name => $prezzo], $storeId);
            } catch (Exception $e) {
                throw new Exception(
                    sprintf(
                        "%s .Problem while processing price. Store Id '%s, Product Ids '%s',  Attribute data '%s'",
                        $e->getMessage(),
                        $storeId,
                        implode(',', $pids),
                        print_r($prezzo, true)
                    )
                );
            }
        }

        return true;
    }

    /**
     * @param $file
     * @return bool|mixed
     */
    public function sendErrorNotifications($file)
    {
        return true;
    }
}
